/* This file is part of MySQLConnectus by Sergio Decherchi

 Copyright (C) 2004 Sergio Decherchi

 This program is free software; you can redistribute it 
 and/or modify it under the terms of the GNU General Public License 
 as published by the Free Software Foundation; either version 2 of 
 the License, or (at your option) any later version.
 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 GNU General Public License for more details (gpl.txt).
 You should have received a copy of the GNU General Public License 
 along with this program; if not, write to the Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */

package dbconnectivity;

import java.io.EOFException;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.Vector;

import javax.swing.JOptionPane;

import tools.OrderedData;

/*Data structure used to buffer a ResultSet.
 * 
 * Here there is the structure of doc , the main Collection of a BufferedResultSet
 * "->>" means "is composed by"
 *
 *  Vector currentDoc -->> 	|Vector page 1 ->>	|Vector line 1	->> 	|String cell 1
 * 			  	    					|Vector page 2			|Vector line 2			|String cell 2
 * 			  							|Vector page 3 		|								.
 * 			      						.								.								|String cell K (K is the number of cells for row)
 * 			  							.								.
 * 										.								.|Vector line M ( effective number of tuples per page, must be <=N)
 * 			  							.
 * 										.
 * 										|Vector page N (every page contains 65 tuples)
 * 
 */

/**
 * This class rapresents a ResultSet that can be : saved using Serializable
 * interface , used without any connection to the dbms.
 * 
 * @version 0.3
 * @author Sergio Decherchi
 */
public class BufferedResultSet implements Serializable {

	/** Document and coulum names containers */
	public Vector doc, columnNames, columnType;

	/** Instance of ordered data */
	private OrderedData od;

	/** Result Set to be saved */
	private ResultSet rs;

	/** Out stream */
	private ObjectOutputStream oos;

	/** In stream */
	private ObjectInputStream ois;

	/** Name of file */
	private String fileName;

	/** Columns number */
	public int cn;

	/** Tuples number */
	public int tn;

	/** Flag for table saved */
	private boolean isSaved = false;

	/** Constructor for output streaming ,starting by a ResultSet */
	public BufferedResultSet(ResultSet rs1, java.io.ObjectOutputStream oos)
			throws IOException {
		ResultSetMetaData rsmd;

		try {
			rsmd = rs.getMetaData();
			cn = rsmd.getColumnCount();
			columnType = new Vector();
			for (int i = 1; i < cn + 1; i++) {
				columnType.add(rsmd.getColumnTypeName(i));
				System.out.println(columnType.get(i - 1));
			}

			tn = od.numberOfTuples;
			write();
		} catch (SQLException sqle) {
			JOptionPane.showMessageDialog(null, "Error in table writing!",
					"Info", 0);
			sqle.printStackTrace();
		}

		rs = rs1;
		this.oos = oos;

	}

	/** Constructor for packing a ResultSet into a BufferedResultSet */
	public BufferedResultSet(ResultSet rs1) {

		rs = rs1;
		columnType = new Vector();
		ResultSetMetaData rsmd;

		try {
			rsmd = rs.getMetaData();
			cn = rsmd.getColumnCount();
			for (int i = 1; i < cn + 1; i++) {
				columnType.add(rsmd.getColumnTypeName(i));
				// System.out.println(rsmd.getColumnTypeName(i));
			}
			setData();
			tn = od.numberOfTuples;
		} catch (SQLException sqle) {
			JOptionPane.showMessageDialog(null, "Error in table writing!",
					"Info", 0);
			sqle.printStackTrace();
		}

	}

	/** Constructor used only for initializing an outputstream */
	public BufferedResultSet(String fileName, ObjectInputStream ois) {

		this.fileName = fileName;
		this.ois = ois;
		read();
	}

	/**
	 * Constructor used for output streaming ,starting by a Vector that contains
	 * data
	 */
	public BufferedResultSet(ObjectOutputStream oos, Vector savedDoc,
			Vector columnNames, int cn, int tn, Vector ct) {
		doc = savedDoc;
		columnType = ct;
		this.columnNames = columnNames;
		this.oos = oos;
		this.cn = cn;
		this.tn = tn;
		isSaved = true;
		try {
			writeObject(oos);
		} catch (IOException ioe) {
			JOptionPane.showMessageDialog(null, "Error in table writing!",
					"Info", 0);
		}

	}

	/** Method used to write ResultSet */
	public void write() {
		try {

			rs.beforeFirst();
			od = new OrderedData();
			od.reorderForStreaming(rs, 65, cn);
			doc = od.doc;
			columnNames = new Vector();
			int column = 0;
			rs.beforeFirst();

			while (column < cn) {

				columnNames.add(((rs.getMetaData()).getColumnName(column + 1)));
				column++;

			}

			rs.beforeFirst();

			writeObject(oos);
		} catch (SQLException sqle2) {
			JOptionPane.showMessageDialog(null, "Error in table writing!",
					"Info", 0);
			sqle2.printStackTrace();
		} catch (IOException ioe) {
			JOptionPane.showMessageDialog(null, "Error in table writing!",
					"Info", 0);
			ioe.printStackTrace();
		}

	}

	/** Used to set data and metadata */
	public void setData() {
		try {

			rs.beforeFirst();
			od = new OrderedData();
			od.reorderForStreaming(rs, 65, cn);
			doc = od.doc;
			columnNames = new Vector();
			int column = 0;
			rs.beforeFirst();

			while (column < cn) {

				columnNames.add(((rs.getMetaData()).getColumnName(column + 1)));
				column++;

			}

			rs.beforeFirst();

		} catch (SQLException sqle2) {
			JOptionPane.showMessageDialog(null, "Error in table writing!",
					"Info", 0);
			sqle2.printStackTrace();
		}

	}

	/** Method used to read stored data */
	public void read() {
		try {
			readObject(ois);
		} catch (Exception ioe) {
			JOptionPane.showMessageDialog(null, "Table reading failed!",
					"Info", 0);
			ioe.printStackTrace();
			return;
		}

	}

	/** Serializable interface implementation */
	private void writeObject(java.io.ObjectOutputStream out) throws IOException {

		if (!isSaved) {
			try {

				rs.beforeFirst();
				//Number of columns
				out.writeInt(cn);
				//Number of tuples
				out.writeInt(tn);

			} catch (SQLException sqle) {
				JOptionPane.showMessageDialog(null, "Table saving failed",
						"Info", 0);
				return;
			}

		} else {
			out.writeInt(cn);
			out.writeInt(tn);
		}

		out.writeObject(doc);
		out.writeObject(columnNames);
		out.writeObject(columnType);
		out.close();
	}

	/** Serializable interface implementation */
	private void readObject(java.io.ObjectInputStream in) throws IOException,
			ClassNotFoundException {

		cn = in.readInt();
		tn = in.readInt();
		doc = (Vector) in.readObject();
		columnNames = (Vector) in.readObject();
		/*
		 * Since version 0.3.2. Try/Catch is used to allow compatibility between
		 * saved tables version <0.3.2. For these saved table import to DB is
		 * not allowed because metadata are not present.
		 */
		try {
			columnType = (Vector) in.readObject();

		} catch (EOFException eof) {
			columnType = null;
		}
		in.close();

	}

}